@@ -21,7 +21,7 @@ import pytest
 from ovirt_imageio_common import directio
 from ovirt_imageio_common import errors
 
-from . import ioutil
+from . import testutil
 
 pytestmark = pytest.mark.skipif(sys.version_info[0] > 2,
                                 reason='needs porting to python 3')
@@ -307,7 +307,7 @@ def test_receive_unbuffered_stream_partial_content(tmpdir):
 def receive_unbuffered(tmpdir, chunks, size, bufsize):
     dst = tmpdir.join("dst")
     dst.write("")
-    src = ioutil.UnbufferedStream(chunks)
+    src = testutil.UnbufferedStream(chunks)
     op = directio.Receive(str(dst), src, size, buffersize=bufsize)
     op.run()
     return dst.read()
@@ -347,6 +347,7 @@ def test_send_no_size(tmpdir, data, offset):
     assert dst.getvalue() == data[offset:]
 
 
+@pytest.mark.parametrize("sparse", [True, False])
 @pytest.mark.parametrize("offset,size", [
     # Aligned offset and size
     (0, directio.BLOCKSIZE),
@@ -371,11 +372,11 @@ def test_send_no_size(tmpdir, data, offset):
     (directio.BUFFERSIZE + 42, directio.BUFFERSIZE - 42),
     (directio.BUFFERSIZE * 2 - 42, 42),
 ])
-def test_zero(tmpdir, offset, size):
+def test_zero(tmpdir, offset, size, sparse):
     dst = tmpdir.join("src")
     data = "x" * directio.BUFFERSIZE * 2
     dst.write(data)
-    op = directio.Zero(str(dst), size, offset=offset)
+    op = directio.Zero(str(dst), size, offset=offset, sparse=sparse)
     op.run()
     with io.open(str(dst), "rb") as f:
         assert f.read(offset) == data[:offset]
@@ -562,7 +563,10 @@ def test_open_no_direct_read_only(tmpdir):
     with io.open(path, "wb") as f:
         f.write(b"x" * 512)
     with directio.open(path, "r", direct=False) as f:
-        assert f.read() == b"x" * 512
+        buf = bytearray(512)
+        n = f.readinto(buf)
+        assert n == 512
+        assert buf == "x" * n
 
 
 def test_open_no_direct_read_write(tmpdir):
@@ -572,7 +576,10 @@ def test_open_no_direct_read_write(tmpdir):
     with directio.open(path, "r+", direct=False) as f:
         f.write(b"b" * 512)
         f.seek(0)
-        assert f.read() == b"b" * 512
+        buf = bytearray(512)
+        n = f.readinto(buf)
+        assert n == 512
+        assert buf == "b" * n
 
 
 def test_open_no_direct_write_only(tmpdir):
